// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KIO_VERSION_H
#define KIO_VERSION_H

#define KIO_VERSION_STRING "6.25.0"
#define KIO_VERSION_MAJOR 6
#define KIO_VERSION_MINOR 25
#define KIO_VERSION_PATCH 0
#define KIO_VERSION ((6<<16)|(25<<8)|(0))

#endif
